/*---------------------------------------------------------------------------*\

Class
    Foam::BLinput

Description
    Bounadry Layer input class
    Extracts B. L. data

\*---------------------------------------------------------------------------*/

#ifndef BLinput_H
#define BLinput_H

#include "fvCFD.H"
#include "scalar.H"
#include "RectangularMatrix.H"
#include "dynamicFvMesh.H"
#include "dynamicMotionSolverFvMesh.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                            Bprime class Declaration
\*---------------------------------------------------------------------------*/

class BLinput
{
    // private

    // 2D case
    RectangularMatrix<scalar> BLinput2DTable_;  // BLinput2DTable
    int indexTime_i_;                           // Index of time inf: time_i >= t
    int indexTime_iPlus1_;                      // Index of time sup: time_i <= t
    scalar currentTimeIndex_;
    scalar f_t_;                                // Time interpolation coefficient
    scalar pw_;
    scalar Tw_;
    scalar v_abl_;
    scalar rhoeUeCH_;
    scalar Hr_;
    scalar qrad_;
    scalar qCFD_;
    scalar heatOn_;
    scalar lambda_;
    scalar Tinf_;
    scalar pressureFactorOn_;
    scalar p_dyn_;

    // 3D non-axisymetric case
    const bool read3DTable_; // flag for reading the 3D table
    fileNameList dirEntries_; // names of the directories which generated the 3D table
    List<scalar> timeEntries_;  // times
    int nVariables_; // number of variables
    int nZones_; // number of zones
    List<string> tecplot_variables_; // name of the variables
    List<List<List<List<scalar> > > > tecplot_variable_values_; // values of the variables
    int tecplotTable_indexPw; // index for the wall pressure
    int tecplotTable_indexRhoe; // index for the density at the edge
    int tecplotTable_indexUe;  // index for the X composant of the velocity at the edge
    int tecplotTable_indexVe; // index for the Y composant of the velocity at the edge
    int tecplotTable_indexWe; // index for the Z composant of the velocity at the edge
    int tecplotTable_indexHe; // index for the enthalpy at the edge
    int tecplotTable_indexCH; // index for the stanton coefficient
    int tecplotTable_indexCoordX; // index for the X coordinate
    int tecplotTable_indexCoordY; // index for the Y coordinate
    int tecplotTable_indexCoordZ; // index for the Z coordinate
    Time& runTime_; // reference to runTime
    scalar currentTime_; // current Time (not used in 3D case)
    PtrList<PtrList<volScalarField> > currentVariables_;  // current volume field of the variables: currentVariables_[variableI][regionI]
    PtrList<dynamicFvMesh>& solidRegions_; // reference to solid regions
    List<fileName> regionNames_; // name of the regions
    List<List<List<scalar> > > currentTimeVariables_; // currentTime for a specific variable varI, region regionI and patch patchI: currentTimeVariables_[varI][regionI][patchI]

public:

    // Constructors

    BLinput();


    //- Construct from components

    BLinput
    (
        RectangularMatrix<scalar>& BLinput2DTable,
        const bool readTecplotTable,
        Time& runTime,
        PtrList<dynamicFvMesh>& solidRegions
    );

    // Destructor

    virtual ~BLinput();

    // Access Functions
    int indexTime_i() const { return indexTime_i_; }
    int indexTime_iPlus1() const { return indexTime_iPlus1_; }
    double f_t() const { return f_t_; }
    scalar pw_i() const { return BLinput2DTable_[indexTime_i_][1]; }
    scalar pw_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][1]; }
    scalar pw() const { return pw_i() * f_t() + pw_iPlus1() * (1 - f_t()); }
    scalar Tw_i() const { return BLinput2DTable_[indexTime_i_][2]; }
    scalar Tw_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][2]; }
    scalar Tw() const { return Tw_i() * f_t() + Tw_iPlus1() * (1 - f_t()); }
    scalar v_abl_i() const { return BLinput2DTable_[indexTime_i_][3]; }
    scalar v_abl_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][3]; }
    scalar v_abl() const { return v_abl_i() * f_t() + v_abl_iPlus1() * (1 - f_t()); }
    scalar rhoeUeCH_i() const { return BLinput2DTable_[indexTime_i_][4]; }
    scalar rhoeUeCH_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][4]; }
    scalar rhoeUeCH() const { return rhoeUeCH_i() * f_t() + rhoeUeCH_iPlus1() * (1 - f_t()); }
    scalar Hr_i() const { return BLinput2DTable_[indexTime_i_][5]; }
    scalar Hr_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][5]; }
    scalar Hr() const { return Hr_i() * f_t() + Hr_iPlus1() * (1 - f_t()); }
    scalar qrad_i() const { return BLinput2DTable_[indexTime_i_][6]; }
    scalar qrad_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][6]; }
    scalar qrad() const { return qrad_i() * f_t() + qrad_iPlus1() * (1 - f_t()); }
    scalar qCFD_i() const { return BLinput2DTable_[indexTime_i_][7]; }
    scalar qCFD_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][7]; }
    scalar qCFD() const { return qCFD_i() * f_t() + qCFD_iPlus1() * (1 - f_t()); }
    scalar heatOn_i() const { return BLinput2DTable_[indexTime_i_][8]; }
    scalar heatOn_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][8]; }
    scalar heatOn() const { return heatOn_i() * f_t() + heatOn_iPlus1() * (1 - f_t()); }
    scalar lambda_i() const { return BLinput2DTable_[indexTime_i_][9]; }
    scalar lambda_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][9]; }
    scalar lambda() const { return lambda_i() * f_t() + lambda_iPlus1() * (1 - f_t()); }
    scalar Tinf_i() const { return BLinput2DTable_[indexTime_i_][10]; }
    scalar Tinf_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][10]; }
    scalar Tinf() const { return Tinf_i() * f_t() + Tinf_iPlus1() * (1 - f_t()); }
    scalar pressureFactorOn_i() const { return BLinput2DTable_[indexTime_i_][11]; }
    scalar pressureFactorOn_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][11]; }
    scalar pressureFactorOn() const { return pressureFactorOn_i() * f_t() + pressureFactorOn_iPlus1() * (1 - f_t()); }
    scalar p_dyn_i() const { return BLinput2DTable_[indexTime_i_][12]; }
    scalar p_dyn_iPlus1() const { return BLinput2DTable_[indexTime_iPlus1_][12]; }
    scalar p_dyn() const { return p_dyn_i() * f_t() + p_dyn_iPlus1() * (1 - f_t()); }

    // Member Functions
    // returns the Temperature interpolation factor and updates the indexes of Ti and Ti+1
    scalar update(scalar Time_);

    // returns the field interpolation for a specific region (from mesh), patch and variable.
    scalarField update(fvMesh& mesh, label patchID, int variableI);

    // returns interpolation scheme for 3D case
    scalarField interpolateEnvironment(fvMesh& mesh, label patchID, int variableI, int indexTime);

    // 2D
    scalar pw(scalar Time_);

    // 3D wall pressure
    scalarField pw(fvMesh& mesh, label patchID);

    // 2D
    scalar Tw(scalar Time_);
    scalar v_abl(scalar Time_);
    scalar rhoeUeCH(scalar Time_);

    // 3D rhoeUeCH
    scalarField rhoeUeCH(fvMesh& mesh, label patchID);
    scalar Hr(scalar Time_);

    // 3D edge enthalpy
    scalarField Hr(fvMesh& mesh, label patchID);
    scalar qrad(scalar Time_);
    scalar qCFD(scalar Time_);
    scalar heatOn(scalar Time_);
    scalar lambda(scalar Time_);
    scalar Tinf(scalar Time_);
    scalar pressureFactorOn(scalar Time_);
    scalar p_dyn(scalar Time_);
}; // End class BLinput
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
