/*---------------------------------------------------------------------------*\

Notices:

    Copyright © 2010 United States Government as represented by the
    Administrator of the National Aeronautics and Space Administration.  All
    Rights Reserved.

Disclaimers

    No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY
    OF ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT
    LIMITED TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO
    SPECIFICATIONS, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
    PARTICULAR PURPOSE, OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE
    SUBJECT SOFTWARE WILL BE ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION,
    IF PROVIDED, WILL CONFORM TO THE SUBJECT SOFTWARE. THIS AGREEMENT DOES
    NOT, IN ANY MANNER, CONSTITUTE AN ENDORSEMENT BY GOVERNMENT AGENCY OR ANY
    PRIOR RECIPIENT OF ANY RESULTS, RESULTING DESIGNS, HARDWARE, SOFTWARE
    PRODUCTS OR ANY OTHER APPLICATIONS RESULTING FROM USE OF THE SUBJECT
    SOFTWARE.  FURTHER, GOVERNMENT AGENCY DISCLAIMS ALL WARRANTIES AND
    LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF PRESENT IN THE ORIGINAL
    SOFTWARE, AND DISTRIBUTES IT "AS IS."

    Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS
    AGAINST THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS,
    AS WELL AS ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT
    SOFTWARE RESULTS IN ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES
    ARISING FROM SUCH USE, INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR
    RESULTING FROM, RECIPIENT'S USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL
    INDEMNIFY AND HOLD HARMLESS THE UNITED STATES GOVERNMENT, ITS CONTRACTORS
    AND SUBCONTRACTORS, AS WELL AS ANY PRIOR RECIPIENT, TO THE EXTENT
    PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR ANY SUCH MATTER SHALL BE
    THE IMMEDIATE, UNILATERAL TERMINATION OF THIS AGREEMENT.

  ---------------------------------------------------------------------------

Application PAM

    PAM is a multi-option sequential solver implementing a generic model for
    multiphase porous materials.

Description of PAM

    PAM is described in Lachaud2017_PATO.pdf provided in directory documentation/publications

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "rhoThermo.H"
#include "turbulentFluidThermoModel.H"
#include "fixedGradientFvPatchFields.H"
#include "regionProperties.H"
#include "compressibleCourantNo.H"
#include "radiationModel.H"
#include "fvOptions.H"
#include "coordinateSystem.H"

// PAM headers

#include "dynamicFvMesh.H"
#include "dynamicMotionSolverFvMesh.H"

#include "IFstream.H"
#include "IOstream.H"
#include "ISstream.H"
#include "BprimeSI.H"
#include "BLinput.H"
#include "materialProperties.H"
#include "gasProperties.H"
#include "RectangularMatrix.H"
#include "fluxFactor.H"

#include "fixedValueFvPatchField.H"
#include "fixedGradientFvPatchField.H"

#include "psiReactionThermo.H"
#include "psiChemistryModel.H"
#include "chemistrySolver.H"
#include "OFstream.H"
#include "thermoPhysicsTypes.H"
#include "basicMultiComponentMixture.H"

#include "probes.H"
#include "IOprobes.H"

#undef Log // conflict between OpenFoam and Mutation++ on the alias "Log" -> undefined here from OpenFoam.
#include <mutation++/mutation++.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "localEulerDdtScheme.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

//    #define NO_CONTROL
//    #define CREATE_MESH createMeshesPostProcess.H
//    #include "postProcess.H"

    #include "setRootCase.H"
    #include "createTime.H"

    regionProperties rp(runTime);

    // set the meshes and read properties for multi-materials
    #include "readSetCase.H"
    #include "createSolidMeshes.H"
    #include "loadMaterialProperties.H"
    #include "createPorousFields.H"         // 1 porous material
    #include "createSubStructureFields.H"   // n substructure materials
    #include "initializeTa.H" // temporary
    #include "createFvOptions.H"
    #include "readGasProperties.H"
    #include "readVirginMaterialThermoTable.H"
    #include "readCharredMaterialThermoTable.H"
    #include "readBprimeTableSI.H"
    #include "readBoundaryConditionProperties.H"
    #include "readMotionProperties.H"
    #include "readChemistryProperties.H"
    #include "setFluxFactorMap.H"
    #include "setWriteOpt.H"
    #include "updateMaterialProperties.H"
    #include "updateGasProperties.H"
    #include "createStreams.H"
    #include "write.H"
    #include "updateTimeZeroDir.H"
    #include "createRDeltaT.H"

if (origin) // Original PAM solver algorythm (kept for tracability and to preserve old tutorial format)
{    
    while (runTime.loop())
    {
        Info << "-----------" << nl << "runTime = " << runTime.timeName()
             << " s ; Time step = " << runTime.deltaT().value() << " s" << endl;

               #include "solid.H"
	       // adaptative time step set on the max pressure gradient.
               #include "setTimeStep_gradp.H"
        
        runTime.write();

        Info << "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
             << "  ClockTime = " << runTime.elapsedClockTime() << " s"
             << nl << endl;
     }
}

else  // PAM-CFD coupled solver (can work with no fluid as well, ie like original PAM)
{
    // set the meshes and read properties for the fluid zones
    #include "createFluidMeshes.H"
    #include "createFluidFields.H"
    #include "initContinuityErrs.H"

    //  from chtMultiRegionFoam
    #include "createTimeControls.H"
    #include "readSolidTimeControls.H"
    #include "compressibleMultiRegionCourantNo.H"
    #include "setInitialMultiRegionDeltaT.H"

    while (runTime.run())
    {

        #include "readTimeControls.H"
        #include "readPIMPLEControls.H"

        #include "compressibleMultiRegionCourantNo.H"
        #include "setMultiRegionDeltaT.H"
	
        runTime++;

        Info << "-----------" << nl << "runTime = " << runTime.timeName()
             << " s ; Time step = " << runTime.deltaT().value() << " s" << endl;

        if (nOuterCorr != 1)
        {
            forAll(fluidRegions, i)
            {
                #include "storeOldFluidFields.H"
            }
        }

       // --- PIMPLE loop
       for (int oCorr=0; oCorr<nOuterCorr; oCorr++)
        {
            bool finalIter = oCorr == nOuterCorr-1;

            forAll(fluidRegions, i)
            {
                Info<< "\nSolving for fluid region "
                    << fluidRegions[i].name() << endl;
                #include "setRegionFluidFields.H"
                #include "readFluidMultiRegionPIMPLEControls.H"
                #include "solveFluid.H"
            }
           
           if (runTime.value() > delaySolid)
           {
           Info << "\nSolving for solid regions " << endl;
               #include "solid.H"
           }
          
        }

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }
}

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
