/*---------------------------------------------------------------------------*\
setEnergyBC.H

Application
    Set the temperature boundary condition on the surface patch.

Description
    There is a switch in the file 'constant/dynamicMeshDict' called:
    convectiveBoundaryLayer.

    - "yes" : CMA-type boundary layer approximation. It needs some simplified
      boundary layer information ('constant/tabulatedData/boundaryLayer') and
      a B' table ('constant/tabulatedData/Bprime'). Numerically, a
      'temperature gradient' boundary condition is used. It uses the
      'gradTempAccess' boundary condition (in libPATO/) that is derived from
      the standard OpenFOAM 'fixedGradient' boundary condition.

    - "no" : reads the motion and the surface temperature directly from file
      'input/motionAndTemperature'. Numerically, a 'temperature' boundary
      condition is used. It uses the standard OpenFOAM 'fixedValue' boundary
      condition.

\*---------------------------------------------------------------------------*/

// restricts to the moving part of the mesh
volScalarField& Tfield =
    const_cast<volScalarField&>
    (
        motionMesh.objectRegistry::lookupObject<volScalarField>
        (
            "Ta"
        )
    );



// access to the right patch of the moving mesh
fixedValueFvPatchScalarField& MovingPatchT =
    refCast<fixedValueFvPatchScalarField>
    (
        Tfield.boundaryFieldRef()[movingPatchID]
    );
scalarField TbcPatchSize(Tw.size());

if (convectiveBoundaryLayer && !volAblation) // if convectiveBoundaryLayer == "yes" (see description in the header)
{
    // Grab T wall
    scalarField pw = p.boundaryField()[movingPatchID];

    // Grab surface normal vectorField to the patch
    vectorField nf =
        - mesh.Sf().boundaryField()[movingPatchID]
        / mesh.magSf().boundaryField()[movingPatchID];

    // Grab the conductivity at the wall
    tensorField kw = k.boundaryField()[movingPatchID];

    // Projection of k on the surface normal
    scalarField kProj = nf & kw & nf;

    // Update the temperature gradient at the wall (explicit)
    scalar Hr                  = BL.Hr(runTime.value());
    scalarField HrTecplotTable = BL.Hr(mesh, movingPatchID);
    scalar qrad                = BL.qrad(runTime.value());
    scalarField BprimeCwBF     = BprimeCw.boundaryField()[movingPatchID];
    scalarField mDotGywBF      = mDotGface.boundaryField()[movingPatchID] & (-nf);
    scalarField hwBF           = hw.boundaryField()[movingPatchID];
    scalarField hgBF           = hg.boundaryField()[movingPatchID];
    scalarField fluxMapw       = fluxMap.boundaryField()[movingPatchID];
    autoPtr<scalarField> rhoeUeCH_ptr;
    if (readBoundaryTecplotTable)
    {
        rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(mesh, movingPatchID)));
    }
    else
    {
        rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(runTime.value())*fluxMapw)); // if modified here, to be modified in setMotion as well
    }
    scalarField& rhoeUeCH = rhoeUeCH_ptr();

    scalar Tbackground    = BL.Tinf(runTime.value());
    scalar lambda         = BL.lambda(runTime.value());
    tau_w                 = tau.boundaryField()[movingPatchID];
    scalarField rho_sw    = rho_s.boundaryField()[movingPatchID];
    scalarField Twint     = Ta.boundaryField()[movingPatchID].patchInternalField();
    scalarField invDxw    = mesh.deltaCoeffs().boundaryField()[movingPatchID];

    // CMA temperature boundary condition used during the heating pulse; for
    // Ref. see NASA CR 1060 (Kendall et al.; 1968)
    if (BL.heatOn(runTime.value()) == 1)
    {
        forAll (TbcPatchSize, i)
        {
            // Blowing correction
            if ((mDotGywBF[i] <= 1e-6) || (rhoeUeCH[i] <= 1e-4))
            {
                BlowingCorrection.boundaryFieldRef()[movingPatchID][i] = 1.0;
            }
            else
            {
                BlowingCorrection.boundaryFieldRef()[movingPatchID][i] =
                    ::log
                    (
                        1. + 2. * lambda *
                        (
                            mDotGywBF[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i])
                            + BprimeCwBF[i]
                        )
                    ) /
                    (
                        2. * lambda *
                        (
                            mDotGywBF[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i])
                            + BprimeCwBF[i]
                        )
                    );
            }

            BprimeGw.boundaryFieldRef()[movingPatchID][i] =
                mDotGywBF[i] /
                (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]);


            if (readBoundaryTecplotTable)
            {
                Hr = HrTecplotTable[i];
            }

            // Wall temperature computed from temperature gradient at the wall (in the solid)
            MovingPatchT[i] =
                Twint[i]
                + (
                    1. / (kProj[i] * invDxw[i]) *
                    (
                        rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i] * (Hr - hwBF[i] * wallSpeciesDiffusion)
                        + mDotGywBF[i] * (hgBF[i] - hwBF[i])
                        + BprimeCwBF[i] * rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i] * (charred.h(pw[i], Tw[i]) - hwBF[i])
                        - (
                            virgin.eps(p[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.eps(p[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * sigmaSB.value() * (pow4(Tw[i]) - pow4(Tbackground))
                        + (
                            virgin.alpha(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.alpha(pw[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * qrad
                    )
                );
        }
    } // if ((BL.heatOn(runTime.value()) == 1))
    else
    {
        // basic re-radiation boundary condition used for cool-down
        forAll (TbcPatchSize, i)
        {
            MovingPatchT[i] =
                Twint[i] +
                (
                    1 / (kProj[i] * invDxw[i]) *
                    (
                        // - mDotGywBF[i]* (hwBF[i]-hgBF[i])
                        - (
                            virgin.eps(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.eps(pw[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * sigmaSB.value() * (pow4(Tw[i]) - pow4(Tbackground))
                        + (
                            virgin.alpha(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.alpha(pw[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * qrad
                    )
                );
        }

    } // if ((BL.heatOn(runTime.value()) == 1))
} // end of "if convectiveBoundaryLayer = "yes" (see description in the header)"

if (!convectiveBoundaryLayer) // if convectiveBoundaryLayer = "no" (see description in the header)
{
    // updating BL values
    scalar Tw = BL.Tw(runTime.value());
    scalarField fluxMapw = fluxMap.boundaryField()[movingPatchID];

    forAll(TbcPatchSize, i)
    {
        MovingPatchT[i] = max(Tw * fluxMapw[i], BL.Tinf(runTime.value()));
    }

} // end of "if convectiveBoundaryLayer = "no" (see description in the header)"

/************************* In-depth finite-rate chemistry (in development) ***********************************/

if (convectiveBoundaryLayer && volAblation) // if convectiveBoundaryLayer = "yes" (see description in the header)
{
    // Grab T wall
    scalarField pw = p.boundaryField()[movingPatchID];

    // Grab surface normal vectorField to the patch
    vectorField nf =
        - mesh.Sf().boundaryField()[movingPatchID]
        / mesh.magSf().boundaryField()[movingPatchID];
    // Grab the conductivity at the wall
    tensorField kw = k.boundaryField()[movingPatchID];
    // Projection of k on the surface normal
    scalarField kProj = nf & kw & nf;

    // Update the temperature gradient at the wall (explicit)
    scalar Hr                  = BL.Hr(runTime.value());
    scalarField HrTecplotTable = BL.Hr(mesh, movingPatchID);
    scalar qrad                = BL.qrad(runTime.value());
    scalarField mDotGywBF      = mDotGface.boundaryField()[movingPatchID] & (-nf);
    scalarField hgBF           = hg.boundaryField()[movingPatchID];
    scalarField hwBF           = hw.boundaryField()[movingPatchID];
    scalarField fluxMapw       = fluxMap.boundaryField()[movingPatchID];
    autoPtr<scalarField> rhoeUeCH_ptr;

    if (readBoundaryTecplotTable)
    {
        rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(mesh, movingPatchID)));
    }
    else
    {
        rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(runTime.value())*fluxMapw)); // if modified here, to be modified in setMotion as well
    }

    scalarField& rhoeUeCH = rhoeUeCH_ptr();
    scalar Tbackground    = BL.Tinf(runTime.value());
    scalar lambda         = BL.lambda(runTime.value());
    tau_w                 = tau.boundaryField()[movingPatchID];
    scalarField rho_sw    = rho_s.boundaryField()[movingPatchID];
    scalarField Twint     = Ta.boundaryField()[movingPatchID].patchInternalField();
    scalarField invDxw    = mesh.deltaCoeffs().boundaryField()[movingPatchID];
    scalarField hwDiffBF    = hwDiff.boundaryField()[movingPatchID];

// CMA temperature boundary condition used during the heating pulse; for Ref. see NASA CR 1060 (Kendall et al.; 1968)
    if ((BL.heatOn(runTime.value()) == 1))
    {
        forAll (TbcPatchSize, i)
        {
            // Blowing correction
            if ((mDotGywBF[i] <= 1e-6) || (rhoeUeCH[i] <= 1e-4))
            {
                BlowingCorrection.boundaryFieldRef()[movingPatchID][i] = 1.0;
            }
            else
            {
                BlowingCorrection.boundaryFieldRef()[movingPatchID][i] =
                    ::log(1 + 2 * lambda * (mDotGywBF[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i])))
                    / (2 * lambda * (mDotGywBF[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i])));
            }

            BprimeGw.boundaryFieldRef()[movingPatchID][i] = mDotGywBF[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]);

            if (readBoundaryTecplotTable)
            {
                Hr = HrTecplotTable[i];
            }

           // compute hew
           pTp[1] = Tw[i];
           pTp[0] = pw[i];
           mixFiniteRate.setState(p_Yie, pTp, 2);
           scalar hew = mixFiniteRate.mixtureHMass();
           /*Info << "hew = " << hew << nl;
           Info << "hw = " << hgBF[i] << nl;
           Info << "Ch* = " << rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i] << nl;
           Info << "Ch* x hew = " << rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i] * hew << nl;
           Info << "Ch* x hw + hwDiffBF = " << rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i] * hgBF[i] + hwDiffBF[i] << nl;        
           */

           // Wall temperature computed from temperature gradient at the wall (in the solid)
            MovingPatchT[i] =  Twint[i] +
                               (
                                   1 / (kProj[i] * invDxw[i]) *
                                   (
                                       rhoeUeCH[i]
                                       * BlowingCorrection.boundaryField()[movingPatchID][i]
                                       * (Hr - hew) // * (Hr - hgBF[i]) + hwDiffBF[i]  equivalent  
                                       - (virgin.eps(p[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c) + charred.eps(p[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))) * sigmaSB.value() * (pow4(Tw[i]) - pow4(Tbackground))
                                       + (virgin.alpha(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c) + charred.alpha(pw[i], Tw[i]) * (1 - tau_w[i] * rho_v / max(rho_sw[i], rho_c))) * qrad
                                   )
                               );
        }
    } // end of if
    else   // basic re-radiation boundary condition used for cool-down
    {
        forAll (TbcPatchSize, i)
        {
            MovingPatchT[i] =
                Twint[i] +
                (
                    1. / (kProj[i] * invDxw[i]) *
                    (
                        // - mDotGywBF[i]* (hwBF[i]-hgBF[i])
                        - (
                            virgin.eps(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.eps(pw[i], Tw[i]) * (1. - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * sigmaSB.value() * (pow4(Tw[i]) - pow4(Tbackground))
                        + (
                            virgin.alpha(pw[i], Tw[i]) * tau_w[i] * rho_v / max(rho_sw[i], rho_c)
                            + charred.alpha(pw[i], Tw[i]) * (1. - tau_w[i] * rho_v / max(rho_sw[i], rho_c))
                        ) * qrad
                    )
                );
        }
    } // end of else */
} // end of "if convectiveBoundaryLayer = "yes" (see description in the header)"
