/*---------------------------------------------------------------------------*\
setEnergyBC.H

Application
    Set the temperature boundary condition on the radiative patch.

Description
    There is a switch in the file 'constant/dynamicMeshDict' called:
    convectiveBoundaryLayer.
    - "yes" : radiative subMat patch active
    - "no" : all subMat patches are internal or bottom

\*---------------------------------------------------------------------------*/

// read patch name
word externalPatchName(dynamicMeshDict_sub[i].lookupOrDefault<word>("externalPatch", "none"));
label externalPatchID =
    solidRegions[i].boundaryMesh().findPatchID(externalPatchName);

if (externalPatchID < 0)
{
    // Info<<"subMat externalPatch ' " << externalPatchName << " ' not found"<<nl;
}
else
{
    Info << "Applying subMat external Patch BC" << nl;
    // restricts to the moving part of the mesh
    volScalarField& Tfield_sub =
        const_cast<volScalarField&>
        (
            solidRegions[i].objectRegistry::lookupObject<volScalarField>
            (
                "Ta"
            )
        );

    // access to the right patch of the moving mesh
    fixedValueFvPatchScalarField& MovingPatchT_sub =
        refCast<fixedValueFvPatchScalarField>
        (
            Tfield_sub.boundaryFieldRef()[externalPatchID]
        );

    scalarField Tw_sub = Ta_sub[i].boundaryField()[externalPatchID];
    scalarField TbcPatchSize(Tw_sub.size());

    if (convectiveBoundaryLayer) // if convectiveBoundaryLayer = "yes" (see description in the header)
    {
        // Grab the conductivity at the wall
        scalarField kw_sub = k_sub[i].boundaryField()[externalPatchID];

        // Update the temperature gradient at the wall (explicit)
        scalar Hr                  = BL.Hr(runTime.value());
        scalarField HrTecplotTable = BL.Hr(solidRegions[i], externalPatchID);
        scalar qrad                = BL.qrad(runTime.value());
        scalarField fluxMapw_sub   = fluxMap_sub[i].boundaryField()[externalPatchID];
        autoPtr<scalarField> rhoeUeCH_ptr;

        if (readBoundaryTecplotTable)
        {
            rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(solidRegions[i], externalPatchID)));
        }
        else
        {
            rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(runTime.value())*fluxMapw_sub));
        }

        scalarField& rhoeUeCH  = rhoeUeCH_ptr();
        scalar Tbackground     = BL.Tinf(runTime.value());
        scalarField Twint_sub  = Ta_sub[i].boundaryField()[externalPatchID].patchInternalField();
        scalarField invDxw_sub = solidRegions[i].deltaCoeffs().boundaryField()[externalPatchID];

        forAll (TbcPatchSize, faceI)
        {
            if (readBoundaryTecplotTable)
            {
                Hr = HrTecplotTable[faceI];
            }

            // Wall temperature computed from temperature gradient at the wall (in the solid)
            MovingPatchT_sub[faceI] =
                Twint_sub[faceI] +
                (
                    1. / (kw_sub[faceI] * invDxw_sub[faceI]) *
                    (
                        rhoeUeCH[faceI] * Hr
                        - 0.9 * sigmaSB.value() * (pow4(Tw_sub[faceI]) - pow4(Tbackground))
                        + 0.9 * qrad
                    )
                );
        }
    }
    else // if convectiveBoundaryLayer = "no" (see description in the header)
    {
        // updating BL values
        scalar Tw = BL.Tw(runTime.value());

        forAll (TbcPatchSize, faceI)
        {
            MovingPatchT_sub[faceI] =
                max(Tw, BL.Tinf(runTime.value()));
            // max(Tw*fluxMapw[i], BL.Tinf(runTime.value()));
        }
    } // end of "if convectiveBoundaryLayer = "no" (see description in the header)"

}
//***** radiation flow coupling

// read patch name
word radiationPatchName(dynamicMeshDict_sub[i].lookupOrDefault<word>("radiationPatch", "none"));
label radiationPatchID =
    solidRegions[i].boundaryMesh().findPatchID(radiationPatchName);

if (radiationPatchID < 0)
{
//     Info<<"subMat radiationPatch ' " << radiationPatchName << " ' not found"<<nl;
}
else
{

    scalarField Tw_sub = Ta_sub[i].boundaryField()[radiationPatchID];
    scalarField radiationPatchSize(Tw_sub.size());
    scalar Tbackground     = BL.Tinf(runTime.value());

    forAll(radiationPatchSize, faceI)
    {
        e_sub[i].boundaryFieldRef()[radiationPatchID][faceI] =
                e_sub_coefs[i][0]
              + Tw_sub[faceI] * e_sub_coefs[i][1]
              + pow(Tw_sub[faceI], 2) * e_sub_coefs[i][2]
              + pow(Tw_sub[faceI], 3) * e_sub_coefs[i][3]
              + pow(Tw_sub[faceI], 4) * e_sub_coefs[i][4];
    }

    scalarField e_sub_w = e_sub[i].boundaryField()[radiationPatchID];
    
    forAll(radiationPatchSize, faceI)
    {
        Qr_sub[i].boundaryFieldRef()[radiationPatchID][faceI] = 
            - e_sub_w[faceI]*sigmaSB.value()* (pow4(Tw_sub[faceI]) - pow4(Tbackground));
    }
    
}

