dynamicFvMesh& mesh = solidRegions[0]; // porous material has to be the first region

//- Reading fields and transport properties
/*******************************************/
// Opening the material property file ...

IOdictionary constantProperties
(
    IOobject
    (
        "data/constantProperties",
        runTime.constant(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

// ...  and reading some universal constants and a few initialisation values
// (they are also described in the run directory in the constantProperties file-- where they are read from)
dimensionedScalar R(constant::physicoChemical::R); // ideal gas constant
dimensionedScalar sigmaSB(constant::physicoChemical::sigma); // Stefan-Boltzmann constant
dimensionedScalar D0(constantProperties.lookup("D0")); // default value for the diffusion coefficient
dimensionedScalar eta0(constantProperties.lookup("eta0")); // default value for the tortuosity

vector vectorX(1, 0, 0);
vector vectorY(0, 1, 0);
vector vectorZ(0, 0, 1);

/*********************************************/

/*  Field variables (primary) */
volScalarField Ta
(
    IOobject
    (
        "Ta",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

volScalarField T
(
    IOobject
    (
        "T",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    Ta
);

volScalarField Tg
(
    IOobject
    (
        "Tg",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("Tg", dimensionSet(0, 0, 0, 1, 0, 0, 0), 0)
);

Tg=Ta; // initialisation of the gas temperature


volScalarField p
(
    IOobject
    (
        "p",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

volScalarField M  // Average molar mass of the gas phase
(
    IOobject
    (
        "M",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, 0, 0, 0, -1, 0, 0), 0.02)
);

/* END: Intensive variables */

// number of solid phases
int nSolidPhases = constantProperties.lookupOrDefault<scalar>("nSolidPhases", 0.0);

if (nSolidPhases < 1)
{
    Info << "The number of solid phases 'nSolidPhases' must be > 0. "
            "If not, just use a CFD solver! ;-)"
         << nl;
    return (0);
}

scalar nPyroReac[nSolidPhases]; // number of pyrolysis reactions for each solid phase
scalar sizeXsi(0);

for (int i = 0; i < nSolidPhases; i++)
{
    nPyroReac[i] =
        constantProperties.lookupOrDefault<scalar>
        (
            "nPyroReac[" + std::to_string(i + 1) + "]",
            0.0
        );
    sizeXsi += nPyroReac[i];
}

PtrList<dimensionedScalar> epsI(nSolidPhases + 1), rhoI(nSolidPhases + 1);
forAll(epsI, i)
{
    epsI.set
    (
        i,
        new dimensionedScalar("epsI", dimless, scalar(0.0))
    );

    rhoI.set
    (
        i,
        new dimensionedScalar
        (
            "rhoI",
            dimensionSet(1, -3, 0, 0, 0, 0, 0),
            scalar(0.0)
        )
    );
}

for (int i = 1; i < nSolidPhases + 1; i++) // the solid phases are defined from value 1 (0 is attributed to the gas).
{
    rhoI[i] =  constantProperties.lookup("rhoI[" + std::to_string(i) + "]");
    Info << rhoI[i] <<  nl;

    epsI[i] =  constantProperties.lookup("epsI[" + std::to_string(i) + "]");
    Info << epsI[i] <<  nl;
}

// gas (i=0)
rhoI[0] = dimensionedScalar("rhoI", dimensionSet(1, -3, 0, 0, 0, 0, 0), scalar(0.0));
epsI[0] = dimensionedScalar("epsI", dimless, scalar(0.0));

// hash table for Xsi (column 0: phase index i; column 1: pyro reac index in phase of index i)
RectangularMatrix<label> hashXsi(sizeXsi, 2);
int nindex = 0;

for (int i = 0; i < nSolidPhases; i++)
{
    for (int j = 0; j < nPyroReac[i]; j++)
    {
        hashXsi[nindex + j][0] = i + 1;
        hashXsi[nindex + j][1] = j + 1;
    }
    nindex += nPyroReac[i];
}

/* Pyrolysis advancements */

volScalarField XsiDefault
(
    IOobject
    (
        "XsiDefault",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

PtrList<volScalarField> Xsi(sizeXsi);

forAll(Xsi, i)
{
    Xsi.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "Xsi[" + std::to_string(hashXsi[i][0]) + "][" +
                    std::to_string(hashXsi[i][1]) + "]",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            XsiDefault
        )
    );
}

/* END:  Pyrolysis advancements  */

/* Pyrolysis reactions parameters */

PtrList<dimensionedScalar>
    Fp(sizeXsi), normFp(sizeXsi), Ap(sizeXsi), Ep(sizeXsi),
    mp(sizeXsi), np(sizeXsi), Tp(sizeXsi), hp(sizeXsi);

forAll(Xsi, i)
{
    Fp.set(i, new dimensionedScalar("1", dimless, 0.0));
    normFp.set(i, new dimensionedScalar("1", dimless, 0.0));
    Ap.set(i, new dimensionedScalar("1", dimensionSet(0, 0, -1, 0, 0, 0, 0), 0.0));
    Ep.set(i, new dimensionedScalar("1", dimensionSet(1, 2, -2, 0, -1, 0, 0), 1.0));
    mp.set(i, new dimensionedScalar("1", dimless, 1.0));
    np.set(i, new dimensionedScalar("1", dimless, 1.0));
    Tp.set(i, new dimensionedScalar("1", dimensionSet(0, 0, 0, 1, 0, 0, 0), 10000.0));
    hp.set(i, new dimensionedScalar("1", dimensionSet(0, 2, -2, 0, 0, 0, 0), 1.0));
}

forAll(Xsi, i)
{
    Fp[i] = constantProperties.lookup("F[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << Fp[i] <<  nl;

    normFp[i] = Fp[i] * 0.;
    Info << normFp[i] <<  nl;

    Ap[i] = constantProperties.lookup("A[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << Ap[i] << nl;

    Ep[i] = constantProperties.lookup("E[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << Ep[i] << nl;

    mp[i] = constantProperties.lookup("m[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << mp[i] << nl;

    np[i] = constantProperties.lookup("n[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << np[i] << nl;

    Tp[i] = constantProperties.lookup("T[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << Tp[i] << nl;

    hp[i] = constantProperties.lookup("h[" + std::to_string(hashXsi[i][0]) + "][" + std::to_string(hashXsi[i][1]) + "]");
    Info << hp[i] << nl;
}

/* END: Pyrolysis reactions parameters */

// AFields are used to define in which zones the pyrolysis reactions take place
// to efficiently handle the Ta>Tp temperature threshold (see pyrolysis.H)

PtrList<volScalarField> AField(sizeXsi);

forAll(Xsi, i)
{
    AField.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "AField[" + std::to_string(i) + "]",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            mesh,
            dimensionedScalar("zero", dimensionSet(0, 0, -1, 0, 0, 0, 0), 0.0)
        )
    );
}

volScalarField Zdefault
(
    IOobject
    (
        "Zdefault",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

volScalarField tau
(
    IOobject
    (
        "tau",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("tau", dimless, 0.0)
);

/* Densities and volume fractions (read from constantProperties file)*/

PtrList<volScalarField> rho(nSolidPhases + 1); // first value (0) used for the gas phase

forAll(rho, i)
{
    rho.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "rho[" + std::to_string(i) + "]",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            mesh,
            rhoI[i]
        )
    );
}

rho[0] = p * M / (R * Ta);
volScalarField rho_g = rho[0];

PtrList<volScalarField> eps(nSolidPhases + 1); // first value (0) used for the gas phase

forAll(eps, i)
{
    eps.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "eps[" + std::to_string(i) + "]",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            mesh,
            epsI[i]
        )
    );
}

eps[0] = 0;

forAll(eps, i)
{
    eps[0] += eps[i];
}

eps[0] = 1. - eps[0];

volScalarField eps_g
(
    IOobject
    (
        "eps_g",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    eps[0]
);

scalar normFp_ = 0.;

forAll(Xsi, i)
{
    normFp_ +=
        Fp[i].value() *
        epsI[hashXsi[i][0]].value() *
        rhoI[hashXsi[i][0]].value();
}

forAll(Xsi, i)
{
    if (normFp_ > 0)
    {
        normFp[i] =
            Fp[i] *
            epsI[hashXsi[i][0]].value() *
            rhoI[hashXsi[i][0]].value() /
            normFp_;
    }
    else
    {
        normFp[i] = 1;
    }
}

forAll(Xsi, i)
{
    tau += normFp[i] * (1 - Xsi[i]);
}

scalar rho_c = 0;

for (int i = 1; i < nSolidPhases + 1; i++)
{
    rho_c += rhoI[i].value() * epsI[i].value();
}

forAll(Xsi, i)
{
    rho_c -=
        Fp[i].value() *
        epsI[hashXsi[i][0]].value() *
        rhoI[hashXsi[i][0]].value();
}

Info << "rho_c = " << rho_c << nl;

volScalarField epsrhog
(
    IOobject
    (
        "epsrhog",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    eps_g*rho_g
);

volScalarField rho_a
(
    IOobject
    (
        "rho_a",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("rho_a", dimensionSet(1, -3, 0, 0, 0, 0, 0), 0.0)
);

forAll(rho, i)
{
    rho_a += rho[i] * eps[i];
}

volScalarField rho_s
(
    IOobject
    (
        "rho_s",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    rho_a - rho[0] * eps[0]
);

volVectorField grad_rho_s
(
    IOobject
    (
        "grad_rho_s",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    fvc::grad(rho_s)
);

scalar rho_v = 0.;
for (int i = 1; i < nSolidPhases + 1; i++)
{
    rho_v += rhoI[i].value() * epsI[i].value();
}

Info << "rho_v = " << rho_v << nl;

volScalarField piTotal
(
    IOobject
    (
        "piTotal",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("piTotal", dimensionSet(1, -3, -1, 0, 0, 0, 0), 0.0)
);

PtrList<volScalarField> pi(sizeXsi);

forAll(pi, i)
{
    pi.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "pi[" + std::to_string(i) + "]",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            piTotal
        )
    );
}

volScalarField omegaHeterogeneousTotal
(
    IOobject
    (
        "omegaHeterogeneousTotal",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar
    (
        "omegaHeterogeneousTotal",
        dimensionSet(1, -3, -1, 0, 0, 0, 0),
        0.0
    )
);

/* END :  Densities and volume fractions */

/*  Effective conductivity */
dimensionedTensor kI
(
    "kI",
    dimensionSet(1, 1, -3, -1, 0, 0, 0),
    tensor(1, 0, 0, 0, 1, 0, 0, 0, 1)
);
dimensionedTensor tP(constantProperties.lookup("tP"));
dimensionedScalar kiCoef_d(constantProperties.lookup("kiCoef"));
dimensionedScalar kjCoef_d(constantProperties.lookup("kjCoef"));
dimensionedScalar kkCoef_d(constantProperties.lookup("kkCoef"));
scalar kiCoef = kiCoef_d.value();
scalar kjCoef = kjCoef_d.value();
scalar kkCoef = kkCoef_d.value();

volTensorField k
(
    IOobject
    (
        "k",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    kI
);

volTensorField kijk
(
    IOobject
    (
        "kijk",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    kI
);

volSymmTensorField k_abl_sym
(
    IOobject
    (
        "k_abl_sym",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    symm(kijk)
);

/*  END: Effective conductivity */

/*  rhoCp */

dimensionedScalar Cp("Cp", dimensionSet(0, 2, -2, -1, 0, 0, 0), 1e3);

volScalarField rhoCp
(
    IOobject
    (
        "rhoCp",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    rho_a * Cp
);

volScalarField cp_g
(
    IOobject
    (
        "cp_g",
        runTime.timeName(),
        mesh
    ),
    mesh,
    Cp
);
/* END:  rhoCp */

/* data for Darcy's Law */
dimensionedScalar eps_g_v(constantProperties.lookup("eps_g_v"));
dimensionedScalar eps_g_c(constantProperties.lookup("eps_g_c"));

volScalarField mu  // viscosity
(
    IOobject
    (
        "mu",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("mu", dimensionSet(1, -1, -1, 0, 0, 0, 0), 2e-5)
);

//- permeability
dimensionedTensor K_v(constantProperties.lookup("K_v"));
dimensionedTensor K_c(constantProperties.lookup("K_c"));

volTensorField K
(
    IOobject
    (
        "K",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    K_v
);

//- Gamma and Beta are explicit quantities used in "gMass.H"
volTensorField Gamma
(
    IOobject
    (
        "Gamma",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    p * M / (mu * R * Ta) * K
);

volScalarField Beta
(
    IOobject
    (
        "Beta",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    eps[0] * M / (R * Ta)
);

/* Derived fields */

volVectorField v
(
    IOobject
    (
        "v",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mesh,
    dimensionedVector("v", dimensionSet(0, 1, -1, 0, 0, 0, 0), vector::zero)
);

volVectorField U
(
    IOobject
    (
        "U",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    v*eps_g
);

volScalarField vi
(
    IOobject
    (
        "vi",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    v.component(0)
);

volScalarField vj
(
    IOobject
    (
        "vj",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    v.component(1)
);

volScalarField vk
(
    IOobject
    (
        "vk",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    v.component(2)
);

volScalarField vmag
(
    IOobject
    (
        "vmag",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mag(v)
);


volVectorField mDotG
(
    IOobject
    (
        "mDotG",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    rho_g * v
);

volScalarField mDotC
(
    IOobject
    (
        "mDotC",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -2, -1, 0, 0, 0, 0), 0.0)
);

surfaceVectorField mDotGface // interpolation of mDotG on the faces
(
    IOobject
    (
        "mDotGface",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    linearInterpolate(mDotG)
);

// Tbc: temperature boundary condition variable - used in "setTemperature.H" to store the surface temperature or surface temperature gradient
volScalarField Tbc
(
    IOobject
    (
        "Tbc",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    Ta
);

volScalarField BprimeCw
(
    IOobject
    (
        "BprimeCw",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);

volScalarField BprimeGw
(
    IOobject
    (
        "BprimeGw",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);

volScalarField hw
(
    IOobject
    (
        "hw",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);

volScalarField hg
(
    IOobject
    (
        "hg",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(0, 2, -2, 0, 0, 0, 0), 0.0)
);

volTensorField GammaHg
(
    IOobject
    (
        "GammaHg",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    hg * p * M / (mu * R * Ta) * K
);

volScalarField hi
(
    IOobject
    (
        "hi",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(0, 2, -2, 0, 0, 0, 0), 0.0)
);

volScalarField h_bar
(
    IOobject
    (
        "h_bar",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(0, 2, -2, 0, 0, 0, 0), 0.0)
);

volScalarField h_c
(
    IOobject
    (
        "h_c",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(0, 2, -2, 0, 0, 0, 0), 0.0)
);

volScalarField Ediff
(
    IOobject
    (
        "Ediff",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, 0, 0, 0, 0), 0.0)
);

volScalarField Ediff0
(
    IOobject
    (
        "Ediff0",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, 0, 0, 0, 0), 0.0)
);

volScalarField hwDiff
(
    IOobject
    (
        "hwDiff",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, 0, -3, 0, 0, 0, 0), 0.0)
);

volScalarField pyrolysisEnergy
(
    IOobject
    (
        "pyrolysisEnergy",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, 0, 0, 0, 0), 0.0)
);

volScalarField omegaHeterogeneousEnergy
(
    IOobject
    (
        "omegaHeterogeneousEnergy",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, 0, 0, 0, 0), 0.0)
);

volScalarField HvT
(
    IOobject
    (
        "HvT",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, 0, 0, 0, 0), 0.0)
);

volScalarField Hv
(
    IOobject
    (
        "Hv",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -3, -1, 0, 0, 0), 0.0)
);

dimensionedScalar Hv0(constantProperties.lookup("Hv0"));
Hv = Hv0;

volScalarField BlowingCorrection
(
    IOobject
    (
        "BlowingCorrection",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);

volScalarField epsgRhogEg
(
    IOobject
    (
        "epsgRhogEg",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, -1, -2, 0, 0, 0, 0), 0.0)
);

volScalarField fluxMap
(
    IOobject
    (
        "fluxMap",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);

volScalarField pressureMap
(
    IOobject
    (
        "pressureMap",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    scalar(1.0)
);
/* END: Derived fields */

/* Surface fields used for verification */
surfaceTensorField kS
(
    IOobject
    (
        "kS",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    linearInterpolate(k)
);

surfaceVectorField phiT
(
    IOobject
    (
        "phiT",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::NO_WRITE
    ),
    kS & (fvc::snGrad(Ta) * mesh.Sf() / mesh.magSf())
);

surfaceVectorField vS
(
    IOobject
    (
        "vS",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    linearInterpolate(v)
);

/*** development ***/
if(!porous)
{
K=0*K;
eps_g=0*eps_g;
}

volVectorField gradP
(
    IOobject
    (
        "gradP",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    fvc::grad(p)
);

Info << "Field allocation is done for the ablative material." << nl;
