using Foam::vector;
dynamicFvMesh& mesh = solidRegions[0]; // porous material has to be the first region

//- Reading fields and transport properties
/*******************************************/
// Opening the transport property file ...
IOdictionary constantProperties
(
    IOobject
    (
        "constantProperties",
        runTime.constant(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

Switch speciesPyrolysis = constantProperties.lookupOrDefault<Switch>("speciesPyrolysis", "no");

// ...  and reading some universal constants and a few initialisation values
// (they are also described in the run directory in the constantProperties file-- where they are read from)
dimensionedScalar R(constantProperties.lookup("R")); // ideal gas constant
dimensionedScalar sigmaPlanck(constantProperties.lookup("sigmaPlanck")); // Planck's constant
dimensionedScalar D0(constantProperties.lookup("D0")); // default value for the diffusion coefficient
dimensionedScalar eta0(constantProperties.lookup("eta0")); // default value for the tortuosity

vector vectorX(1, 0, 0);
vector vectorY(0, 1, 0);
vector vectorZ(0, 0, 1);
vector vector0(0, 0, 0);

/*********************************************/

/*  Field variables (primary) */

wordList list_BC(mesh.boundaryMesh().size());
forAll(list_BC,patchI)
{
  list_BC[patchI]="fixedValue";
}
wordList list_BC_zeroGrad(mesh.boundaryMesh().size());
forAll(list_BC_zeroGrad,patchI)
{
  list_BC_zeroGrad[patchI]="zeroGradient";
}

volScalarField rhoByRhov
(
 IOobject
 (
  "rhoByRhov",
  runTime.timeName(),
  mesh,
  IOobject::NO_READ,
  IOobject::AUTO_WRITE
  ),
 mesh,
 dimensionedScalar("zero", dimensionSet(1, -3, 0, 0, 0, 0, 0), 0.0),
 list_BC
 );


volScalarField Ta
(
    IOobject
    (
        "Ta",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);


volScalarField p
(
    IOobject
    (
        "p",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

volScalarField M  // Average molar mass of the gas phase
(
    IOobject
    (
        "M",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mesh,
    dimensionedScalar("zero", dimensionSet(1, 0, 0, 0, -1, 0, 0), 0.02),
    list_BC_zeroGrad
);

/* END: Intensive variables */

// number of solid phases
int nSolidPhases = constantProperties.lookupOrDefault<scalar>("nSolidPhases", 0.0);

if (nSolidPhases < 1)
{
    Info << "The number of solid phases 'nSolidPhases' must be > 0. "
            "If not, just use a CFD solver! ;-)"
         << nl;
    return (0);
}

scalar nPyroReac[nSolidPhases]; // number of pyrolysis reactions for each solid phase
std::stringstream sstrmCabl;
scalar sizeXsi(0);

for (int i = 0; i < nSolidPhases; i++)
{
    sstrmCabl << "nPyroReac[" << i + 1 << "]";
    nPyroReac[i] =
        constantProperties.lookupOrDefault<scalar>(sstrmCabl.str().c_str(), 0.0);
    sstrmCabl.str("");
    sizeXsi += nPyroReac[i];
}

PtrList<dimensionedScalar> epsI(nSolidPhases + 1), rhoI(nSolidPhases + 1);
forAll(epsI, i)
{
    epsI.set
    (
        i,
        new dimensionedScalar("epsI", dimless, scalar(0.0))
    );

    rhoI.set
    (
        i,
        new dimensionedScalar
        (
            "rhoI",
            dimensionSet(1, -3, 0, 0, 0, 0, 0),
            scalar(0.0)
        )
    );
}

std::stringstream sstrmSol;
for (int i = 1; i < nSolidPhases + 1; i++) // the solid phases are defined from value 1 (0 is attributed to the gas).
{
    sstrmSol << "rhoI[" << i << "]";
    rhoI[i] =  constantProperties.lookup(sstrmSol.str().c_str());
    Info << rhoI[i] <<  nl;
    sstrmSol.str("");
    sstrmSol << "epsI[" << i << "]";
    epsI[i] =  constantProperties.lookup(sstrmSol.str().c_str());
    Info << epsI[i] <<  nl;
    sstrmSol.str("");
}

// gas (i=0)
rhoI[0] = dimensionedScalar("rhoI", dimensionSet(1, -3, 0, 0, 0, 0, 0), scalar(0.0));
epsI[0] = dimensionedScalar("epsI", dimless, scalar(0.0));

// hash table for Xsi (column 0: phase index i; column 1: pyro reac index in phase of index i)
RectangularMatrix<scalar> hashXsi(sizeXsi, 2);
int nindex = 0;

for (int i = 0; i < nSolidPhases; i++)
{
    for (int j = 0; j < nPyroReac[i]; j++)
    {
        hashXsi[nindex + j][0] = i + 1;
        hashXsi[nindex + j][1] = j + 1;
    }
    nindex += nPyroReac[i];
}

/* Pyrolysis advancements */

volScalarField XsiDefault
(
    IOobject
    (
        "XsiDefault",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

PtrList<volScalarField> Xsi(sizeXsi);
std::stringstream sstrmXsi;

forAll(Xsi, i)
{
    sstrmXsi << "Xsi[" << hashXsi[i][0] << "]" << "[" << hashXsi[i][1] << "]";
    Xsi.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                sstrmXsi.str().c_str(),
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            XsiDefault
        )
    );
    sstrmXsi.str("");
}

/* END:  Pyrolysis advancements  */

/* Pyrolysis reactions parameters */

PtrList<dimensionedScalar>
    Fp(sizeXsi), normFp(sizeXsi), Ap(sizeXsi), Ep(sizeXsi),
    mp(sizeXsi), np(sizeXsi), Tp(sizeXsi), hp(sizeXsi);

forAll(Xsi, i)
{
    Fp.set(i, new dimensionedScalar("1", dimless, 0.0));
    normFp.set(i, new dimensionedScalar("1", dimless, 0.0));
    Ap.set(i, new dimensionedScalar("1", dimensionSet(0, 0, -1, 0, 0, 0, 0), 0.0));
    Ep.set(i, new dimensionedScalar("1", dimensionSet(1, 2, -2, 0, -1, 0, 0), 1.0));
    mp.set(i, new dimensionedScalar("1", dimless, 1.0));
    np.set(i, new dimensionedScalar("1", dimless, 1.0));
    Tp.set(i, new dimensionedScalar("1", dimensionSet(0, 0, 0, 1, 0, 0, 0), 10000.0));
    hp.set(i, new dimensionedScalar("1", dimensionSet(0, 2, -2, 0, 0, 0, 0), 1.0));
}

forAll(Xsi, i)
{
    sstrmXsi << "F[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    Fp[i] = constantProperties.lookup(sstrmXsi.str().c_str());
    Info << Fp[i] <<  nl;

    normFp[i] = Fp[i] * 0.;
    Info << normFp[i] <<  nl;

    sstrmXsi.str("");
    sstrmXsi << "A[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    Ap[i] = (constantProperties.lookup(sstrmXsi.str().c_str()));
    Info << Ap[i] << nl;

    sstrmXsi.str("");
    sstrmXsi << "E[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    Ep[i] =        constantProperties.lookup(sstrmXsi.str().c_str());
    Info << Ep[i] << nl;

    sstrmXsi.str("");
    sstrmXsi << "m[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    mp[i] = constantProperties.lookup(sstrmXsi.str().c_str());
    Info << mp[i] << nl;

    sstrmXsi.str("");
    sstrmXsi << "n[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    np[i] = constantProperties.lookup(sstrmXsi.str().c_str());
    Info << np[i] << nl;

    sstrmXsi.str("");
    sstrmXsi << "T[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    Tp[i] = constantProperties.lookup(sstrmXsi.str().c_str());
    Info << Tp[i] << nl;

    sstrmXsi.str("");
    sstrmXsi << "h[" << hashXsi[i][0] << "]"
             <<  "[" << hashXsi[i][1] << "]";
    hp[i] = constantProperties.lookup(sstrmXsi.str().c_str());
    Info << hp[i] << nl;

    sstrmXsi.str("");
}

/* END: Pyrolysis reactions parameters */

// AFields are used to define in which zones the pyrolysis reactions take place
// to efficiently handle the Ta>Tp temperature threshold (see pyrolysis.H)

PtrList<volScalarField> AField(sizeXsi);
std::stringstream sstrmAField;
forAll(Xsi, i)
{
    sstrmAField << "AField[" << i << "]";

    AField.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                sstrmAField.str().c_str(),
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            mesh,
            dimensionedScalar("zero", dimensionSet(0, 0, -1, 0, 0, 0, 0), 0.0)
        )
    );

    sstrmAField.str("");
}

volScalarField un // blanc dimensionless volume scalar Field equal to 1 everywhere (conveniently used as a multiplier to pass the value of a scalar to the whole mesh -- see examples below)
(
    IOobject
    (
        "un",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

volScalarField Zdefault
(
    IOobject
    (
        "Zdefault",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    ),
    mesh
);

volScalarField tau
(
    IOobject
    (
        "tau",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("tau", dimless, 0.0)
);

/* Densities and volume fractions (read from constantProperties file)*/

PtrList<volScalarField> rho(nSolidPhases + 1); // first value (0) used for the gas phase
std::stringstream sstrmRho;

forAll(rho, i)
{
    sstrmRho << "rho[" << i << "]";
    rho.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                sstrmRho.str().c_str(),
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            rhoI[i] * un
        )
    );

    sstrmRho.str("");
}


dimensionedScalar zero_rho("zero", dimensionSet(1, -3, 0, 0, 0, 0, 0), 0.0);
rho[0] = zero_rho*un; // p * M / (R * Ta);
volScalarField rho_g = rho[0];

PtrList<volScalarField> eps(nSolidPhases + 1); // first value (0) used for the gas phase
std::stringstream sstrmEps;

forAll(eps, i)
{
    sstrmEps << "eps[" << i << "]";
    eps.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                sstrmEps.str().c_str(),
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            epsI[i] * un
        )
    );

    sstrmEps.str("");
}

eps[0] = 0;

forAll(eps, i)
{
    eps[0] += eps[i];
}

eps[0] = 1. - eps[0];


scalar normFp_ = 0.;

forAll(Xsi, i)
{
    normFp_ +=
        Fp[i].value() *
        epsI[hashXsi[i][0]].value() *
        rhoI[hashXsi[i][0]].value();
}

forAll(Xsi, i)
{
    if (normFp_ > 0)
    {
        normFp[i] =
            Fp[i] *
            epsI[hashXsi[i][0]].value() *
            rhoI[hashXsi[i][0]].value() /
            normFp_;
    }
    else
    {
        normFp[i] = 1;
    }
}

forAll(Xsi, i)
{
    tau += normFp[i] * (1 - Xsi[i]);
}

scalar rho_c = 0;

for (int i = 1; i < nSolidPhases + 1; i++)
{
    rho_c += rhoI[i].value() * epsI[i].value();
}

forAll(Xsi, i)
{
    rho_c -=
        Fp[i].value() *
        epsI[hashXsi[i][0]].value() *
        rhoI[hashXsi[i][0]].value();
}

Info << "rho_c = " << rho_c << nl;



volScalarField rho_a
(
    IOobject
    (
        "rho_a",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    mesh,
    dimensionedScalar("rho_a", dimensionSet(1, -3, 0, 0, 0, 0, 0), 0.0)
);

forAll(rho, i)
{
    rho_a += rho[i] * eps[i];
}

volScalarField rho_s
(
    IOobject
    (
        "rho_s",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    rho_a - rho[0] * eps[0]
);

rho_s.write();


scalar rho_v = 0.;
for (int i = 1; i < nSolidPhases + 1; i++)
{
    rho_v += rhoI[i].value() * epsI[i].value();
}

Info << "rho_v = " << rho_v << nl;

volScalarField piTotal
(
    IOobject
    (
        "piTotal",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("piTotal", dimensionSet(1, -3, -1, 0, 0, 0, 0), 0.0)
);

PtrList<volScalarField> pi(sizeXsi);
std::stringstream sstrmPI;
forAll(pi, i)
{
    sstrmPI << "pi[" << i << "]";
    word name_pi = sstrmPI.str().c_str() ;
    name_pi+="_reaction";
    pi.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                name_pi,
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            piTotal
        )
    );
    sstrmPI.str("");
}

// Virgin solid density
dimensionedScalar rho_sv("rho_sv",dimensionSet(1, -3, 0, 0, 0, 0, 0),scalar(0.0) );

for (int i = 1; i < nSolidPhases + 1; i++)
{
  rho_sv += rhoI[i] *epsI[i];
}
