// Initialisation of chemistry operators for OpenFoam and Mutation++.
// The comments below give an overview of the initialisation and interfacing sequences.
// However, a fair knowledge of both code structures is obvisouly required to understand what follows in detail.

Info << "Initialization of chemistry operators and data\n" << endl;

// Read the chemistry property input directory for the ablative material (in OpenFoam)
IOdictionary chemistryProperties
(
    IOobject
    (
        "chemistryProperties",
        runTime.constant(),
        mesh, // runTime.db(),
        IOobject::MUST_READ,
        IOobject::NO_WRITE
    )
);

// Initialize finite-rate chemistry properties using OpenFoam finite-rate chemistry modules
Info << nl << "Reading thermophysicalProperties" << endl;
autoPtr<psiChemistryModel> pChemistry(psiChemistryModel::New(mesh));

psiChemistryModel& chemistry = pChemistry();
psiReactionThermo& thermo = chemistry.thermo();
thermo.validate(args.executable(), "h");

basicMultiComponentMixture& composition = thermo.composition();
PtrList<volScalarField>& Y = composition.Y(); // mass fraction of the species


// Initialize Mutation++ to compute equilibrium chemistry, thermodynamics and transport properties
// Initialization of Mutation++ for equilibrium chemistry
word mixtureMutationEquilibrium (thermo.lookup("mixtureMutationEquilibrium"));
Mutation::MixtureOptions optEq(mixtureMutationEquilibrium);
optEq.setStateModel("Equil");
Mutation::Mixture mix(optEq);

const int ne_mix = mix.nElements();

// Read pyrolysis-production SPECIES mass fractions - nb: only species present in the chemistry mechanism are looked up.
RectangularMatrix<scalar> Yp(sizeXsi, Y.size());
std::stringstream sstrmYpij;
for (int i = 0; i < sizeXsi; i++)
{
    for (int j = 0; j < Y.size(); j++)
    {
        sstrmYpij << "gamma[" << hashXsi[i][0] << "][" << hashXsi[i][1] << "]["
                  << Y[j].name() << "]";
        Yp[i][j] = constantProperties.lookupOrDefault<scalar>(
            sstrmYpij.str().c_str(), 0.0);
        sstrmYpij.str("");
    }
}

// Read pyrolysis-production ELEMENT mass fractions - nb: only elements present
// in the equilibrium file (Mutation) are looked up.
RectangularMatrix<scalar> Zp(sizeXsi, ne_mix);
std::stringstream sstrmZpij;
for (int i = 0; i < sizeXsi; i++)
{
    for (int j = 0; j < ne_mix; j++)
    {
        sstrmZpij << "zeta[" << hashXsi[i][0] << "][" << hashXsi[i][1] << "]["
                  << mix.elementName(j) << "]";
        Zp[i][j] = constantProperties.lookupOrDefault<scalar>(
            sstrmZpij.str().c_str(), 0.0);
        sstrmZpij.str("");
    }
}

// pi_elem_species
int size_pi_elem_species = ne_mix;
if(speciesPyrolysis)
{
  size_pi_elem_species =  Y.size();
}

PtrList<volScalarField> pi_elem_species(size_pi_elem_species);
forAll(pi_elem_species, i)
{
  word name_pi_elem_species="";
  if(speciesPyrolysis)
{
  name_pi_elem_species ="pi_" + Y[i].name();
}
  else
{
  name_pi_elem_species ="pi_" +  mix.elementName(i) ;
}
  pi_elem_species.set
  (
   i,
    new volScalarField
   (
        IOobject
        (
         name_pi_elem_species,
         runTime.timeName(),
         mesh,
         IOobject::NO_READ,
         IOobject::AUTO_WRITE
         ),
        piTotal
    )
   );
}

// write pi in folder 0
{
  forAll(pi,j)
{
  pi[j].write();
}
  forAll(pi_elem_species,i)
{
  pi_elem_species[i].write();
}
  piTotal.write();
}
