/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
  \\    /   O peration     |
  \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
  \\/     M anipulation  |
  -------------------------------------------------------------------------------
  License
  This file is part of OpenFOAM.

  OpenFOAM is free software: you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License
  along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

  Application
  TecDatToFoam

  Description
  Reads Tecplot data (.dat) from BLAYER (DPLR utility) and then creates an input table for PATO (constant/tecplotTable)

  \*---------------------------------------------------------------------------*/

// include headers
#include "SortableList.H"
#include "argList.H"
#include "Time.H"
#include "IFstream.H"
#include "OFstream.H"
#include "pointField.H"
#include "unitConversion.H"
#include "OSspecific.H"
#include "IStringStream.H"
#include "readTecplotFile.H"

// namespace
using namespace Foam;

// function to transform string to decimal 
template<typename T>
T stringToDecimal(const string& s)
{
  T t = T();
  std::stringstream ss(s);
  ss >> t;
  return t;
}

// verify if a string has the suffix
bool has_suffix(const std::string &str, const std::string &suffix)
{
  return str.size() >= suffix.size() &&
    str.compare(str.size() - suffix.size(), suffix.size(), suffix) == 0;
}
// verify if the string "str" begins by the "beginning" argument
bool has_beginning(const std::string &str, const std::string &beginning)
{
  return str.size() >= beginning.size() &&
    str.compare(0, beginning.size(), beginning) == 0;
}
// get the time from a file name
word get_time_file(const std::string &str, const std::string &suffix, const std::string &beginning)
{
  std::string time_string=str.substr(beginning.size()+1,str.size()-suffix.size()-beginning.size()-1);
  word time_word = time_string;
  return time_word;
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
  // Add options and arguments
  argList::noParallel();
  argList::addOption
    (
     "nVar","int","number of variables in dat file."
     );
  argList::addOption
    (
     "nZone","int","number of zones in dat file."
     );

  argList::validArgs.append("dat file");

  argList args(argc, argv);

  if (!args.check())
    {
      FatalError.exit();
    }

  // create times
#include "createTime.H"

  int nVar; // number of variables in the tecplot file (.dat)
  if(!args.optionReadIfPresent("nVar", nVar))
    {
      Info << "Error: you must use option -nVar" << endl;
      return 0;
    }

  int nZone; // number of zones in the tecplot file (.dat)
  if(!args.optionReadIfPresent("nZone", nZone))
    {
      Info << "Error: you must use option -nZone" << endl;
      return 0;
    }

  // verify the data folder exists
  if(!isDir("constant/environmentProperties/tecplotData"))
    {
      Info << "Tecplot data directory (tecplotData/) missing in constant/environmentProperties/" << nl;
      return (0);
    }

  Info << "Reading files in data folder." << endl;
  fileNameList dirFiles(readDir("./constant/environmentProperties/tecplotData", fileName::FILE)); // list of files in data folder
  fileNameList dirEntriesNonSorted; // time folder names
  word dataName = args.args()[1].c_str(); // first argument = beginning of the file names
  SortableList<double> timeEntries; // times from the tecplot files  

  Info << "Files found:" << endl;
  forAll(dirFiles,i)
    {
      if(has_suffix(dirFiles[i],".dat"))
	{
	  if(has_beginning(dirFiles[i],dataName))
	    {
	      dirEntriesNonSorted.append(dirFiles[i]);
	      double time_ = stringToDecimal<double>(get_time_file(dirFiles[i],".dat",dataName)); // get the time from the file name
	      timeEntries.append(time_);    
	    }
	}
    }

  timeEntries.sort(); // sort the times 

  // sort dirEntries 
  fileNameList dirEntries(dirEntriesNonSorted.size());
  int nEntries=0;
  while(nEntries<dirEntriesNonSorted.size())
    {
      forAll(dirEntriesNonSorted,i)
	{
	  double time_ = stringToDecimal<double>(get_time_file(dirEntriesNonSorted[i],".dat",dataName));
	  if(time_==timeEntries[nEntries])
	    {
	      dirEntries[nEntries]=dirEntriesNonSorted[i];
	      
	      nEntries++;
	      break;
	    }
	}
    }
  
  forAll(dirEntries,i)
    {
      Info << " " << dirEntries[i]<< ": time = " << timeEntries[i] << endl;
    }

  Info << endl;
  readTecplotFile rdTec; // readTecplotFile is defined in the same folder
  List<List<List<List<double> > > > var_values(timeEntries.size()); // values of the variables from the tecplot files (var_values[timeI][zoneI][varI])
  List<string> variables(nVar);
  
  // read the tecplot files and fill the variables table
  forAll(dirEntries,i)
    {
      rdTec.readTecFile("constant/environmentProperties/tecplotData/"+dirEntries[i],nVar,nZone,i,var_values,variables);
    }
  
  // PATO input table 
  IOdictionary tecplotTablesIO
    (
     IOobject
     (
      "constant/environmentProperties/tecplotTable",
      runTime,
      IOobject::NO_READ,
      IOobject::NO_WRITE,
      false
      )
     );

  // Write the different variables and times in the PATO input table
  dictionary tecplotTable;
  tecplotTable.set("files", dirEntries);
  tecplotTable.set("times", timeEntries);
  tecplotTable.set("numberZones", nZone);
  tecplotTable.set("numberVariables", nVar);
  tecplotTable.set("variableNames", variables);
  tecplotTable.set("variableValues", var_values);
  tecplotTablesIO.set("tecplotTable" , tecplotTable);
  tecplotTablesIO.regIOobject::write();

  // Read PATO input table 
  IOdictionary tecplotTablesIO_
    (
     IOobject
     (
      "constant/environmentProperties/tecplotTable",
      runTime,
      IOobject::MUST_READ,
      IOobject::NO_WRITE,
      false
      )
     );

  dictionary tecplotTable_ = tecplotTablesIO_.subDict("tecplotTable"); // sub dictionary

  fileNameList dirEntries_(tecplotTable_.lookup("files")); // file names
  List<scalar> timeEntries_(tecplotTable_.lookup("times")); // times
  int nVariables_ =readScalar(tecplotTable_.lookup("numberVariables")); // number of variables
  int nZones_ =readScalar(tecplotTable_.lookup("numberZones")); // number of zones
  List<string> variables_(tecplotTable_.lookup("variableNames")); // variables names
  List<List<List<List<scalar> > > > variable_values_(tecplotTable_.lookup("variableValues")); // variables values
  
  // Print the tecplot files info
  Info << "Read Tecplot table" << endl;
  Info << "files: " ;
  forAll(dirEntries_,i)
    {
      Info << dirEntries_[i] << " ";
    }
  Info << endl;
  Info << "times: " ;
  forAll(timeEntries_,i)
    {
      Info << timeEntries_[i] << " ";
    }
  Info << endl;
  Info << "number of zones = " << nZones_ << " , number of variables = "  << nVariables_ << endl;
  Info << "variables: " ;
   forAll(variables_,i)
    {
        Info << variables_[i] << " ";
    }
  Info << endl;

  
    return 0;
}


// ************************************************************************* //
